@@ -42,7 +42,7 @@ class _AddEventWebServer(BaseHTTPRequestHandler):
         self.send_header("Content-type", "text/html")
         self.end_headers()
 
-    def do_GET(self):
+    def do_GET(self):  # pylint: disable=invalid-name
         """Handle a GET request
 
         This function is used to provide an easy way to check if the webserver is up and running
@@ -55,7 +55,7 @@ class _AddEventWebServer(BaseHTTPRequestHandler):
         else:
             self._set_not_found_headers()
 
-    def do_POST(self):
+    def do_POST(self):  # pylint: disable=invalid-name
         """Handle a POST request
 
         :return: None
@@ -79,7 +79,7 @@ class _AddEventWebServer(BaseHTTPRequestHandler):
         else:
             self._set_not_found_headers()
 
-    def log_message(self, format: str, *args: Any) -> None:
+    def log_message(self, format: str, *args: Any) -> None:  # pylint: disable=redefined-builtin
         # Override the logging to stop it spamming the test output, it's not useful anyway,
         # for example:
         #   127.0.0.1 - - [24/Jul/2022 09:21:54] "POST /add HTTP/1.1" 200 -
@@ -94,7 +94,7 @@ def run_webserver(queue: multiprocessing.Queue, ip_address: str, port: int):
     :param port: The port the webserver should listen on
     :return: None
     """
-    global _EVENT_QUEUE
+    global _EVENT_QUEUE  # pylint: disable=global-statement
     _EVENT_QUEUE = queue
     server_address = (ip_address, port)
     httpd = HTTPServer(server_address, _AddEventWebServer)
