@@ -4,11 +4,16 @@ The test event store and run loop
 import copy
 import multiprocessing
 from multiprocessing import connection
-from typing import Dict, List, Union
+from typing import Dict, List, Union, Tuple
 import logging
+import traceback
 
-from .expectation import Expectation
-from .constants import NO_MATCHING_EVENT_FOUND, EVENT_ID_KEY, EVENT_STATE_KEY
+from .expectation import Expectation, FailedExpectation, ExpectationFailureType, PartialMatch
+from .constants import (
+    NO_MATCHING_EVENT_FOUND,
+    EVENT_ID_KEY,
+    EVENT_STATE_KEY,
+)
 from .messages import *
 
 
@@ -56,7 +61,9 @@ class _TestEventStore:
         self._full_store = _FullEventStore()
 
     def _test_if_event_matches_expectations(
-        self, event: Dict, expectations: List[Expectation]
+        self,
+        event: Dict,
+        expectations: List[Expectation],
     ) -> bool:
         for expectation in expectations:
             if expectation.event_field_name not in event:
@@ -86,7 +93,7 @@ class _TestEventStore:
 
         :return: A list of events
         """
-        logging.debug(f"Event store get_all called, store: {self._store}")
+        logging.debug("Event store get_all called, store: {}".format(self._store))
         return self._store
 
     def get_full_event_log(self) -> List[Dict]:
@@ -94,7 +101,9 @@ class _TestEventStore:
 
         :return: A list of events
         """
-        logging.debug(f"Event store get_full_event_log called, store: {self._full_store.get_all()}")
+        logging.debug(
+            "Event store get_full_event_log called, store: {}".format(self._full_store.get_all())
+        )
         return self._full_store.get_all()
 
     def get_all_matching(self, expectations: List[Expectation]) -> List[Dict]:
@@ -124,8 +133,8 @@ class _TestEventStore:
         self._store.append(test_event)
 
         logging.debug(
-            f"Event store add called, event: {test_event},"
-            f" there are now {len(self._store)} events in the store"
+            "Event store add called, event: {},"
+            " there are now {} events in the store".format(test_event, len(self._store))
         )
 
         self._full_store.add_event(test_event)
@@ -168,7 +177,7 @@ class _TestEventStore:
             del self._store[index]
             self._full_store.mark_event_as_deleted(event["event_id"])
 
-        logging.debug(f"{num_events_removed} events removed from the store")
+        logging.debug("{} events removed from the store".format(num_events_removed))
         return num_events_removed
 
     def find_first_match(self, expectations: List[Expectation]) -> Union[str, Dict]:
@@ -185,6 +194,65 @@ class _TestEventStore:
 
         return NO_MATCHING_EVENT_FOUND
 
+    def extended_expect_event(
+        self,
+        expectations: List[Expectation],
+        partial_filter_field_expectations: List[Expectation],
+    ) -> Tuple[Union[str, Dict], List[PartialMatch]]:
+        """Find an event matching the expectations, if no event is found, details of events that
+        partially match and the expectations that failed will be returned.
+
+        :param expectations: A list of expectations
+        :param partial_filter_fields: Which fields should be used to perform the partial
+          filtering if any
+        :return: A tuple of the matching event or an error message as well as a list of events
+          that partially match with expectation failures
+        """
+        events_matching_partial_filter = []
+        for event in self._store:
+            match = self._test_if_event_matches_expectations(
+                event, partial_filter_field_expectations
+            )
+            if not match:
+                continue
+            events_matching_partial_filter.append(event)
+
+            match = self._test_if_event_matches_expectations(event, expectations)
+            if match:
+                return (event, [])
+
+        partial_match_event_info = []
+        if len(events_matching_partial_filter) > 0:
+            # Go through the events where the partial filter fields match and check
+            # which expectations failed
+            for event in events_matching_partial_filter:
+                partial_match_event_info.append(
+                    PartialMatch(event, self._build_expectation_failures(event, expectations))
+                )
+
+        return (NO_MATCHING_EVENT_FOUND, partial_match_event_info)
+
+    def _build_expectation_failures(
+        self, event: Dict, expectations: List[Expectation]
+    ) -> List[FailedExpectation]:
+        expectation_failures = []
+
+        for expectation in expectations:
+            if expectation.event_field_name not in event:
+                expectation_failures.append(
+                    FailedExpectation(expectation, ExpectationFailureType.FIELD_NOT_FOUND)
+                )
+                continue
+
+            event_value = event[expectation.event_field_name]
+            if not expectation.comparison_function(event_value, expectation.comparison_value):
+                expectation_failures.append(
+                    FailedExpectation(
+                        expectation, ExpectationFailureType.COMPARISON_FAILURE, event_value
+                    )
+                )
+        return expectation_failures
+
 
 def event_store_run_loop(queue: multiprocessing.Queue, result_pipe: connection.Connection):
     """The event store process run loop
@@ -222,11 +290,6 @@ def event_store_run_loop(queue: multiprocessing.Queue, result_pipe: connection.C
             elif message_type == GetFullEventLogMessage.TYPE:
                 result_pipe.send(event_store.get_full_event_log())
 
-            elif message_type == GetEventMessage.TYPE:
-                expectations = message.expectations
-                result = event_store.find_first_match(expectations)
-                result_pipe.send(result)
-
             elif message_type == GetAllMatchingEventsMessage.TYPE:
                 expectations = message.expectations
                 result_pipe.send(event_store.get_all_matching(expectations))
@@ -240,6 +303,14 @@ def event_store_run_loop(queue: multiprocessing.Queue, result_pipe: connection.C
                 result = event_store.find_first_match(expectations)
                 result_pipe.send(result)
 
+            elif message_type == ExpectEventExtendedDebugMessage.TYPE:
+                expectations = message.expectations
+                partial_filter_field_expectations = message.partial_filter_field_expectations
+                extended_result = event_store.extended_expect_event(
+                    expectations, partial_filter_field_expectations
+                )
+                result_pipe.send(extended_result)
+
             elif message_type == RemoveExpectedEventMessage.TYPE:
                 del_event = message.event
                 event_store.remove_expected_event(del_event)
@@ -249,7 +320,11 @@ def event_store_run_loop(queue: multiprocessing.Queue, result_pipe: connection.C
                 events_removed = event_store.remove_matching_events(expectations)
                 result_pipe.send(events_removed)
             else:
-                logging.error(f"Event Store - Unhandled message: {message_type}")
-    except Exception as exc:
-        logging.error(f"An unhandled exception has occurred in the event store: {exc}")
+                logging.error("Event Store - Unhandled message: {}".format(message_type))
+    except Exception:
+        logging.error(
+            "An unhandled exception has occurred in the event store: {}".format(
+                traceback.format_exc()
+            )
+        )
     result_pipe.close()
